/**
 * apeman react package for image component.
 * @constructor ApImage
 */

"use strict";


import React, {PropTypes as types} from 'react';
import ReactDOM from 'react-dom';
import classnames from 'classnames';
import extend from 'extend';
import numcal from 'numcal';
import _scaledSize from './_scaled_size';
import {ApSpinner} from 'apeman-react-spinner';

/** @lends ApImage */
let ApImage = React.createClass({


    //--------------------
    // Specs
    //--------------------

    propTypes: {
        scale: types.oneOf([
            'fit',
            'fill',
            'none'
        ]),
        width: types.number,
        height: types.number,
        src: types.string,
        alt: types.string,
        spinnerTheme: types.string,
        onLoad: types.func,
        onError: types.func
    },

    mixins: [],

    statics: {
        scaledSize: _scaledSize,
        zeroIfNaN(value) {
            return isNaN(value) ? 0 : value;
        }
    },

    getInitialState() {
        let s = this;
        return {
            imgWidth: null,
            imgHeight: null,
            loading: !!s.props.src,
            error: null
        };
    },

    getDefaultProps() {
        return {
            scale: 'none',
            width: null,
            height: null,
            src: null,
            alt: "NO IMAGE",
            spinnerTheme: 'a',
            onLoad: null,
            onError: null
        };
    },

    render() {
        let s = this,
            state = s.state,
            props = s.props;

        let size = {
            width: props.width || 'auto',
            height: props.height || 'auto'
        };

        return (
            <div className={classnames('ap-image', props.className, {
                'ap-image-loading': props.src && state.loading
            })}
                 style={extend({}, size, props.style)}>
                {state.error ? s._renderNotfound(size) : s._renderImg(size)}
                {state.loading ? s._renderSpinner(size) : ''}
            </div>
        );
    },


    //--------------------
    // Lifecycle
    //--------------------

    componentWillMount() {
        let s = this;
    },

    componentDidMount() {
        let s = this;
    },

    componentWillReceiveProps(nextProps) {
        let s = this;

        let src = s.props.src,
            nextSrc = nextProps.src;
        s.setState({
            loading: !!nextSrc && (nextSrc !== src)
        });
    },

    shouldComponentUpdate(nextProps, nextState) {
        let s = this;
        return true;
    },

    componentWillUpdate(nextProps, nextState) {
        let s = this;
    },

    componentDidUpdate(prevProps, prevState) {
        let s = this;
    },

    componentWillUnmount() {
        let s = this;
    },

    //------------------
    // Helper
    //------------------

    handleLoad(e) {
        let s = this,
            {props} = s;

        let elm = ReactDOM.findDOMNode(s);

        let frameSize = {
                width: elm.offsetWidth,
                height: elm.offsetHeight
            },
            contentSize = {
                height: e.target.height,
                width: e.target.width
            },
            scaledSize = ApImage.scaledSize(
                contentSize, frameSize, props.scale
            );

        s.setState({
            imgWidth: scaledSize.width,
            imgHeight: scaledSize.height,
            loading: false
        });

        if (props.onLoad) {
            props.onLoad(e);
        }
    },

    handleError(e) {
        let s = this,
            {props} = s;

        s.setState({
            error: e,
            loading: false
        });

        if (props.onError) {
            props.onError(e);
        }
    },

    //------------------
    // Private
    //------------------
    _renderImg(size) {
        let s = this,
            state = s.state,
            props = s.props;

        let zeroIfNaN = ApImage.zeroIfNaN;

        return (
            <img src={props.src}
                 alt={props.alt}
                 className={classnames('ap-image-content')}
                 style={{
                        top:zeroIfNaN((size.height - state.imgHeight) / 2),
                        left:zeroIfNaN((size.width - state.imgWidth) / 2),
                        width:state.imgWidth,
                        height:state.imgHeight
                     }}
                 onLoad={s.handleLoad}
                 onError={s.handleError}/>
        );
    },
    _renderNotfound(size) {
        let s = this,
            props = s.props;

        return (
            <div className="ap-image-notfound"
                 style={{
                    lineHeight:`${size.height}px`,
                    fontSize:`${numcal.min(size.height * 0.5, 24)}`
                 }}
            >{props.alt}</div>
        );
    },
    _renderSpinner(size) {
        let s = this,
            props = s.props;
        return (
            <ApSpinner className="ap-image-spinner"
                       theme={props.spinnerTheme}
                       style={{
                       width: size.width,
                       height:size.height
                       }}/>
        );
    }
});

module.exports = ApImage;